#include "PasswordDialog.h"

#include <QLineEdit>
#include <QPushButton>
#include <QVBoxLayout>
#include <QByteArray>
#include <QCryptographicHash>
#include <QtDebug>
#include <QLabel>

PasswordDialog::PasswordDialog( const QString &aPassword, const QString &aText, const QString &aTitle, QWidget *aParent ) :
		QDialog( aParent )
{
	setWindowTitle( aTitle.isNull() ? tr( "Enter password" ) : aTitle );
	setModal( true );

	iPasswordHash = aPassword.toUtf8();

	/* Create the layout:

	.--------title---------.
	| text                 |
	|                      |
	| [***_______________] |
	| [Cancel]      [ OK ] |
	'----------------------'
	*/

	QVBoxLayout *layout = new QVBoxLayout;

	iText = new QLabel( aText );
	layout->addWidget( iText );
	layout->addStretch();

	iPasswordEdit = new QLineEdit;
	iPasswordEdit->setEchoMode( QLineEdit::Password );
	layout->addWidget( iPasswordEdit );

	QHBoxLayout *buttonLayout = new QHBoxLayout;
	QPushButton *buttonOK = new QPushButton( tr( "OK" ) );
	QPushButton *buttonCancel = new QPushButton( tr( "Cancel" ) );
	
	buttonLayout->addStretch();
	buttonLayout->addWidget( buttonOK );
	buttonLayout->addWidget( buttonCancel );
	buttonLayout->addStretch();
	layout->addSpacing( 5 );
	layout->addLayout( buttonLayout );

	// Connect the user action signals to corresponding slots
	connect( buttonOK, SIGNAL( pressed() ), this, SLOT( okButtonPressed() ) );
	connect( buttonCancel, SIGNAL( pressed() ), this, SLOT( cancelButtonPressed() ) );

	// Enable the layout
	setLayout( layout );
}

PasswordDialog::~PasswordDialog()
{
}

void PasswordDialog::okButtonPressed()
{
	qDebug() << "PasswordDialog::okButtonPressed()";

	// Get md5 hash from the password entered to the dialog
	QCryptographicHash *hash = new QCryptographicHash( QCryptographicHash::Md5 );
	hash->addData( iPasswordEdit->text().toUtf8() );
	QByteArray userpw = hash->result();
	delete hash;
	
	close();
		
	// Compare the password hashes and emit corresponding signal tellin if the password was correct
	if ( iPasswordHash == userpw.toHex() )
	{
		emit passwordEntered( PasswordDialog::Correct );
		qDebug() << "Password OK";
	}
	else
	{
		emit passwordEntered( PasswordDialog::Incorrect );
		qDebug() << "Incorrect password!";
	}
}

void PasswordDialog::cancelButtonPressed()
{
	qDebug() << "PasswordDialog::cancelButtonPressed()";
	
	close();
	emit passwordEntered( PasswordDialog::Canceled );
}

void PasswordDialog::update( const QString &aText )
{
	qDebug() << "PasswordDialog::update()";
	iText->setText( aText );
}
