#include <QtTest/QtTest>
#include "Configuration.h"
#include "ConnectionSettings.h"
#include "TestCommunication.h"
#include "Communication.h"
#include "MessagingUtils.h"
#include <QString>
#include <QSignalSpy>

static const QString server = "192.168.0.35";
static const QString user   = "maemo";
static const QString pass   = "P@ssw0rd";
static const unsigned int refresh = 1;

void TestCommunication::initTestCase()
{
    ConnectionSettings *conn = new ConnectionSettings( server, user, pass, refresh );
    QVERIFY( conn != 0 );
	iComm = new Communication( *conn );
	QVERIFY( iComm != 0 );
    delete conn;
    conn = NULL;
	connect( iComm,
			 SIGNAL( requestFinished( int, int ) ),
			 this,
			 SLOT( requestFinished( int, int ) )
			);
}

void TestCommunication::cleanupTestCase()
{
	delete iComm;
}

void TestCommunication::init() { }

void TestCommunication::cleanup() { }

void TestCommunication::testPublicSignals()
{
	QSignalSpy rp( iComm, SIGNAL(readProgress( int, int, int )) );
	QCOMPARE( rp.isValid(), true );
	QSignalSpy rs( iComm, SIGNAL(requestStarted( int )) );
	QCOMPARE( rs.isValid(), true );
	QSignalSpy rf( iComm, SIGNAL(requestFinished( int, int )) );
	QCOMPARE( rf.isValid(), true );
	
	iRequestId = iComm->request( "", QString("").toAscii() );
	for(int i=0; iRequestId != 0 && i < 5000/250; i++ )
		QTest::qWait( 250 );
	
	//verify requestStarted
	QCOMPARE( rs.count(),  1 );
	QList<QVariant> args = rs.takeFirst();
	QCOMPARE( args.at(0).type(), QVariant::Int );
	
	//verify requestFinished
	QCOMPARE( rf.count(), 1 );
	args = rf.takeFirst();
	QCOMPARE( args.at(0).type(), QVariant::Int );
	QCOMPARE( args.at(1).type(), QVariant::Int );
	
	//verify readProgress
	QCOMPARE( rp.count(), 1 );
	args = rp.takeFirst();
	QCOMPARE( args.at(0).type(), QVariant::Int );
	QCOMPARE( args.at(1).type(), QVariant::Int );
	QCOMPARE( args.at(2).type(), QVariant::Int );
}

void TestCommunication::testRequestAndResponse()
{
	const QString command = "text/xml; charset=utf-8; action=\"http://schemas.microsoft.com/exchange/services/2006/messages/GetUserAvailability\"";
    QFile file("input.xml");
    if(!file.open(QIODevice::ReadOnly))
    {
    	QFAIL( "Error opening input.xml" );
    }
    QByteArray content = file.readAll();

	iError = 0;
	int id_ok = iComm->request( command, content );
	iRequestId = id_ok; 
	int id_fail = iComm->request( "", QString("").toAscii() );
	QVERIFY( iRequestId != 0 );
	QVERIFY( id_fail != 0 );
	qDebug() << "id_ok:" << id_ok;
	qDebug() << "id_fail: " << id_fail;
	for(int i=0; iRequestId != 0 && i < 5000/250; i++ )
		QTest::qWait( 250 );
	qDebug() << "iError:" << iError;
	QVERIFY( iError == 0 );
	QByteArray* response = iComm->response( id_ok );
	QVERIFY( response != NULL );
	QVERIFY( !response->isEmpty() );
	if( response->contains("GetUserAvailabilityResponse") == FALSE )
	{
		QWARN( *response );
	}
	QVERIFY( response->contains("GetUserAvailabilityResponse") );
	delete response;
	response = iComm->response( 0 );
	QVERIFY( response == NULL );
}

void TestCommunication::requestFinished( int aRequestId, int aError )
{
	qDebug() << "TestCommunication::requestFinished:" << aRequestId << aError;
	if( aRequestId == iRequestId )
		iError = aError;
	iRequestId = 0;
}

